@@ -190,14 +190,17 @@ server_connect_copy_skeleton(SERVER_CONNECT_REC *src, int connect_info)
 	dest->away_reason = g_strdup(src->away_reason);
 	dest->no_autojoin_channels = src->no_autojoin_channels;
 	dest->no_autosendcmd = src->no_autosendcmd;
+	dest->unix_socket = src->unix_socket;
 
-	dest->use_ssl = src->use_ssl;
-	dest->ssl_cert = g_strdup(src->ssl_cert);
-	dest->ssl_pkey = g_strdup(src->ssl_pkey);
-	dest->ssl_verify = src->ssl_verify;
-	dest->ssl_cafile = g_strdup(src->ssl_cafile);
-	dest->ssl_capath = g_strdup(src->ssl_capath);
-	dest->ssl_ciphers = g_strdup(src->ssl_ciphers);
+	dest->use_tls = src->use_tls;
+	dest->tls_cert = g_strdup(src->tls_cert);
+	dest->tls_pkey = g_strdup(src->tls_pkey);
+	dest->tls_verify = src->tls_verify;
+	dest->tls_cafile = g_strdup(src->tls_cafile);
+	dest->tls_capath = g_strdup(src->tls_capath);
+	dest->tls_ciphers = g_strdup(src->tls_ciphers);
+	dest->tls_pinned_cert = g_strdup(src->tls_pinned_cert);
+	dest->tls_pinned_pubkey = g_strdup(src->tls_pinned_pubkey);
 
 	return dest;
 }
