@@ -167,7 +167,6 @@ static void server_connect_callback_init(SERVER_REC *server, GIOChannel *handle)
 	server_connect_finished(server);
 }
 
-#ifdef HAVE_OPENSSL
 static void server_connect_callback_init_ssl(SERVER_REC *server, GIOChannel *handle)
 {
 	int error;
@@ -198,7 +197,6 @@ static void server_connect_callback_init_ssl(SERVER_REC *server, GIOChannel *han
 
 	server_connect_finished(server);
 }
-#endif
 
 static void server_real_connect(SERVER_REC *server, IPADDR *ip,
 				const char *unix_socket)
@@ -221,7 +219,7 @@ static void server_real_connect(SERVER_REC *server, IPADDR *ip,
 		own_ip = IPADDR_IS_V6(ip) ? server->connrec->own_ip6 : server->connrec->own_ip4;
 		port = server->connrec->proxy != NULL ?
 			server->connrec->proxy_port : server->connrec->port;
-		handle = server->connrec->use_ssl ?
+		handle = server->connrec->use_tls ?
 			net_connect_ip_ssl(ip, port, own_ip, server) : net_connect_ip(ip, port, own_ip);
 	} else {
 		handle = net_connect_unix(unix_socket);
@@ -239,7 +237,7 @@ static void server_real_connect(SERVER_REC *server, IPADDR *ip,
 			}
 			server->no_reconnect = TRUE;
 		}
-		if (server->connrec->use_ssl && errno == ENOSYS)
+		if (server->connrec->use_tls && errno == ENOSYS)
 			server->no_reconnect = TRUE;
 
 		server->connection_lost = TRUE;
@@ -247,11 +245,9 @@ static void server_real_connect(SERVER_REC *server, IPADDR *ip,
 		g_free(errmsg2);
 	} else {
 		server->handle = net_sendbuffer_create(handle, 0);
-#ifdef HAVE_OPENSSL
-		if (server->connrec->use_ssl)
+		if (server->connrec->use_tls)
 			server_connect_callback_init_ssl(server, handle);
 		else
-#endif
 		server->connect_tag =
 			g_input_add(handle, G_INPUT_WRITE | G_INPUT_READ,
 				    (GInputFunction)
@@ -626,22 +622,24 @@ void server_connect_unref(SERVER_CONNECT_REC *conn)
 	g_free_not_null(conn->own_ip4);
 	g_free_not_null(conn->own_ip6);
 
-        g_free_not_null(conn->password);
-        g_free_not_null(conn->nick);
-        g_free_not_null(conn->username);
+	g_free_not_null(conn->password);
+	g_free_not_null(conn->nick);
+	g_free_not_null(conn->username);
 	g_free_not_null(conn->realname);
 
-	g_free_not_null(conn->ssl_cert);
-	g_free_not_null(conn->ssl_pkey);
-	g_free_not_null(conn->ssl_pass);
-	g_free_not_null(conn->ssl_cafile);
-	g_free_not_null(conn->ssl_capath);
-	g_free_not_null(conn->ssl_ciphers);
+	g_free_not_null(conn->tls_cert);
+	g_free_not_null(conn->tls_pkey);
+	g_free_not_null(conn->tls_pass);
+	g_free_not_null(conn->tls_cafile);
+	g_free_not_null(conn->tls_capath);
+	g_free_not_null(conn->tls_ciphers);
+	g_free_not_null(conn->tls_pinned_cert);
+	g_free_not_null(conn->tls_pinned_pubkey);
 
 	g_free_not_null(conn->channels);
-        g_free_not_null(conn->away_reason);
+	g_free_not_null(conn->away_reason);
 
-        conn->type = 0;
+	conn->type = 0;
 	g_free(conn);
 }
 
@@ -684,21 +682,11 @@ SERVER_REC *cmd_options_get_server(const char *cmd,
 				   SERVER_REC *defserver)
 {
 	SERVER_REC *server;
-	GSList *list, *tmp, *next;
+	GList *list;
 
 	/* get all the options, then remove the known ones. there should
 	   be only one left - the server tag. */
-	list = hashtable_get_keys(optlist);
-	if (cmd != NULL) {
-		for (tmp = list; tmp != NULL; tmp = next) {
-			char *option = tmp->data;
-			next = tmp->next;
-
-			if (command_have_option(cmd, option))
-				list = g_slist_remove(list, option);
-		}
-	}
-
+	list = optlist_remove_known(cmd, optlist);
 	if (list == NULL)
 		return defserver;
 
@@ -713,7 +701,7 @@ SERVER_REC *cmd_options_get_server(const char *cmd,
 		server = NULL;
 	}
 
-	g_slist_free(list);
+	g_list_free(list);
 	return server;
 }
 
