@@ -40,6 +40,9 @@
 #include <string.h>
 #include <signal.h>
 
+/* After LINE_SPLIT_LIMIT characters, the message will be split into multiple lines */
+#define LINE_SPLIT_LIMIT 400
+
 typedef void (*ENTRY_REDIRECT_KEY_FUNC) (int key, void *data, SERVER_REC *server, WI_ITEM_REC *item);
 typedef void (*ENTRY_REDIRECT_ENTRY_FUNC) (const char *line, void *data, SERVER_REC *server, WI_ITEM_REC *item);
 
@@ -70,6 +73,7 @@ static int paste_timeout_id;
 static int paste_use_bracketed_mode;
 static int paste_bracketed_mode;
 static int paste_was_bracketed_mode;
+static int previous_yank_preceded;
 
 /* Terminal sequences that surround the input when the terminal has the
  * bracketed paste mode active. Fror more details see
@@ -227,7 +231,7 @@ static void paste_buffer_join_lines(GArray *buf)
 	}
 
 	/* all looks fine - now remove the whitespace, but don't let lines
-	   get longer than 400 chars */
+	   get longer than LINE_SPLIT_LIMIT chars */
 	dest = arr; last_lf = TRUE; last_lf_pos = NULL; line_len = 0;
 	for (i = 0; i < buf->len; i++) {
 		if (last_lf && isblank(arr[i])) {
@@ -245,7 +249,7 @@ static void paste_buffer_join_lines(GArray *buf)
 			last_lf = TRUE;
 		} else {
 			last_lf = FALSE;
-			if (++line_len >= 400 && last_lf_pos != NULL) {
+			if (++line_len >= LINE_SPLIT_LIMIT && last_lf_pos != NULL) {
 				memmove(last_lf_pos+1, last_lf_pos,
 					(dest - last_lf_pos) * sizeof(unichar));
 				*last_lf_pos = '\n'; last_lf_pos = NULL;
@@ -357,11 +361,24 @@ static void insert_paste_prompt(void)
 {
 	char *str;
 
+	/* The actual number of lines that will show up post-line-split */
+	int actual_line_count = paste_line_count;
+	int split_lines = paste_buffer->len / LINE_SPLIT_LIMIT;
+
+	/* in case this prompt is happening due to line-splitting, calculate the
+	   number of lines obtained from this. The number isn't entirely accurate;
+	   we just choose the greater of the two since the exact value isn't
+	   important */
+	if (split_lines > paste_verify_line_count &&
+		split_lines > paste_line_count) {
+		actual_line_count = split_lines;
+	}
+
 	paste_prompt = TRUE;
 	paste_old_prompt = g_strdup(active_entry->prompt);
 	printformat_window(active_win, MSGLEVEL_CLIENTNOTICE,
 			   TXT_PASTE_WARNING,
-			   paste_line_count,
+			   actual_line_count,
 			   active_win->active == NULL ? "window" :
 			   active_win->active->visible_name);
 
@@ -424,11 +441,20 @@ static void sig_gui_key_pressed(gpointer keyp)
 		gui_entry_insert_char(active_entry, key);
 		ret = 1;
 	} else {
+		previous_yank_preceded = active_entry->yank_preceded;
+		active_entry->yank_preceded = FALSE;
+		active_entry->previous_append_next_kill = active_entry->append_next_kill;
+		active_entry->append_next_kill = FALSE;
 		ret = key_pressed(keyboard, str);
 		if (ret < 0) {
 			/* key wasn't used for anything, print it */
 			gui_entry_insert_char(active_entry, key);
 		}
+		if (ret == 0) {
+			/* combo not complete, ignore append_next_kill and yank_preceded */
+			active_entry->append_next_kill = active_entry->previous_append_next_kill;
+			active_entry->yank_preceded = previous_yank_preceded;
+		}
 	}
 
 	/* ret = 0 : some key create multiple characters - we're in the middle
@@ -547,7 +573,7 @@ static void key_forward_to_space(void)
 static void key_erase_line(void)
 {
 	gui_entry_set_pos(active_entry, active_entry->text_len);
-	gui_entry_erase(active_entry, active_entry->text_len, TRUE);
+	gui_entry_erase(active_entry, active_entry->text_len, CUTBUFFER_UPDATE_REPLACE);
 }
 
 static void key_erase_to_beg_of_line(void)
@@ -555,7 +581,7 @@ static void key_erase_to_beg_of_line(void)
 	int pos;
 
 	pos = gui_entry_get_pos(active_entry);
-	gui_entry_erase(active_entry, pos, TRUE);
+	gui_entry_erase(active_entry, pos, CUTBUFFER_UPDATE_PREPEND);
 }
 
 static void key_erase_to_end_of_line(void)
@@ -564,7 +590,7 @@ static void key_erase_to_end_of_line(void)
 
 	pos = gui_entry_get_pos(active_entry);
 	gui_entry_set_pos(active_entry, active_entry->text_len);
-	gui_entry_erase(active_entry, active_entry->text_len - pos, TRUE);
+	gui_entry_erase(active_entry, active_entry->text_len - pos, CUTBUFFER_UPDATE_APPEND);
 }
 
 static void key_yank_from_cutbuffer(void)
@@ -574,7 +600,32 @@ static void key_yank_from_cutbuffer(void)
         cutbuffer = gui_entry_get_cutbuffer(active_entry);
 	if (cutbuffer != NULL) {
 		gui_entry_insert_text(active_entry, cutbuffer);
-                g_free(cutbuffer);
+		active_entry->yank_preceded = TRUE;
+		g_free(cutbuffer);
+	}
+}
+
+static void key_yank_next_cutbuffer(void)
+{
+	GUI_ENTRY_CUTBUFFER_REC *rec;
+	guint length = 0;
+	char *cutbuffer;
+
+	if (!previous_yank_preceded)
+		return;
+
+	if (active_entry->kill_ring == NULL)
+		return;
+
+	rec = active_entry->kill_ring->data;
+	if (rec != NULL) length = rec->cutbuffer_len;
+
+	cutbuffer = gui_entry_get_next_cutbuffer(active_entry);
+	if (cutbuffer != NULL) {
+		gui_entry_erase(active_entry, length, CUTBUFFER_UPDATE_NOOP);
+		gui_entry_insert_text(active_entry, cutbuffer);
+		active_entry->yank_preceded = TRUE;
+		g_free(cutbuffer);
 	}
 }
 
@@ -611,34 +662,44 @@ static void key_delete_character(void)
 
 static void key_backspace(void)
 {
-	gui_entry_erase(active_entry, 1, FALSE);
+	gui_entry_erase(active_entry, 1, CUTBUFFER_UPDATE_NOOP);
 }
 
 static void key_delete_previous_word(void)
 {
-	gui_entry_erase_word(active_entry, FALSE);
+	gui_entry_erase_word(active_entry, FALSE, CUTBUFFER_UPDATE_PREPEND);
 }
 
 static void key_delete_next_word(void)
 {
-	gui_entry_erase_next_word(active_entry, FALSE);
+	gui_entry_erase_next_word(active_entry, FALSE, CUTBUFFER_UPDATE_APPEND);
 }
 
 static void key_delete_to_previous_space(void)
 {
-	gui_entry_erase_word(active_entry, TRUE);
+	gui_entry_erase_word(active_entry, TRUE, CUTBUFFER_UPDATE_PREPEND);
 }
 
 static void key_delete_to_next_space(void)
 {
-	gui_entry_erase_next_word(active_entry, TRUE);
+	gui_entry_erase_next_word(active_entry, TRUE, CUTBUFFER_UPDATE_APPEND);
+}
+
+static void key_append_next_kill(void)
+{
+	active_entry->append_next_kill = TRUE;
 }
 
 static gboolean paste_timeout(gpointer data)
 {
+	int split_lines;
 	paste_was_bracketed_mode = paste_bracketed_mode;
 
-	if (paste_line_count == 0) {
+	/* number of lines after splitting extra-long messages */
+	split_lines = paste_buffer->len / LINE_SPLIT_LIMIT;
+
+	/* Take into account the fact that a line may be split every LINE_SPLIT_LIMIT characters */
+	if (paste_line_count == 0 && split_lines <= paste_verify_line_count) {
 		int i;
 
 		for (i = 0; i < paste_buffer->len; i++) {
@@ -647,8 +708,9 @@ static gboolean paste_timeout(gpointer data)
 		}
 		g_array_set_size(paste_buffer, 0);
 	} else if (paste_verify_line_count > 0 &&
-		   paste_line_count >= paste_verify_line_count &&
-		   active_win->active != NULL)
+				(paste_line_count >= paste_verify_line_count ||
+				split_lines > paste_verify_line_count) &&
+				active_win->active != NULL)
 		insert_paste_prompt();
 	else
 		paste_flush(TRUE);
@@ -1170,6 +1232,8 @@ void gui_readline_init(void)
 	key_bind("erase_to_beg_of_line", "Erase everything before the cursor", NULL, NULL, (SIGNAL_FUNC) key_erase_to_beg_of_line);
 	key_bind("erase_to_end_of_line", "Erase everything after the cursor", "^K", NULL, (SIGNAL_FUNC) key_erase_to_end_of_line);
 	key_bind("yank_from_cutbuffer", "\"Undelete\", paste the last deleted text", "^Y", NULL, (SIGNAL_FUNC) key_yank_from_cutbuffer);
+	key_bind("yank_next_cutbuffer", "Revert to the previous last deleted text", NULL, NULL, (SIGNAL_FUNC) key_yank_next_cutbuffer);
+	key_bind("append_next_kill", "Append next deletion", NULL, NULL, (SIGNAL_FUNC) key_append_next_kill);
 	key_bind("transpose_characters", "Swap current and previous character", "^T", NULL, (SIGNAL_FUNC) key_transpose_characters);
 	key_bind("transpose_words", "Swap current and previous word", NULL, NULL, (SIGNAL_FUNC) key_transpose_words);
 	key_bind("capitalize_word", "Capitalize the current word", NULL, NULL, (SIGNAL_FUNC) key_capitalize_word);
@@ -1259,6 +1323,8 @@ void gui_readline_deinit(void)
 	key_unbind("erase_to_beg_of_line", (SIGNAL_FUNC) key_erase_to_beg_of_line);
 	key_unbind("erase_to_end_of_line", (SIGNAL_FUNC) key_erase_to_end_of_line);
 	key_unbind("yank_from_cutbuffer", (SIGNAL_FUNC) key_yank_from_cutbuffer);
+	key_unbind("yank_next_cutbuffer", (SIGNAL_FUNC) key_yank_next_cutbuffer);
+	key_unbind("append_next_kill", (SIGNAL_FUNC) key_append_next_kill);
 	key_unbind("transpose_characters", (SIGNAL_FUNC) key_transpose_characters);
 	key_unbind("transpose_words", (SIGNAL_FUNC) key_transpose_words);
 
